@@ -276,7 +276,7 @@ class Transformer(BaseAgent):
         self.logger.debug("trigger_release_inputs, updated_contents: %s" % str(updated_contents))
         return updated_contents
 
-    def trigger_release_inputs(self, updated_output_contents, work, input_output_maps):
+    def trigger_release_inputs(self, updated_output_contents, work, input_output_maps, final=False):
         to_release_inputs = {}
         for map_id in input_output_maps:
             outputs = input_output_maps[map_id]['outputs'] if 'outputs' in input_output_maps[map_id] else []
@@ -288,7 +288,7 @@ class Transformer(BaseAgent):
                     to_release_inputs[content['coll_id']].append(content)
 
         # updated_contents = core_transforms.release_inputs(to_release_inputs)
-        updated_contents = core_transforms.release_inputs_by_collection(to_release_inputs)
+        updated_contents = core_transforms.release_inputs_by_collection(to_release_inputs, final=final)
         self.logger.debug("trigger_release_inputs, to_release_inputs: %s" % str(to_release_inputs))
         self.logger.debug("trigger_release_inputs, updated_contents: %s" % str(updated_contents))
         return updated_contents
@@ -853,7 +853,7 @@ class Transformer(BaseAgent):
             self.logger.info("Transform(%s) work is terminated, trigger to release all final status files" % (transform['transform_id']))
             if work.use_dependency_to_release_jobs():
                 self.logger.info("trigger_release_inputs: %s" % transform['transform_id'])
-                to_release_input_contents1 = self.trigger_release_inputs(updated_output_contents_full, work, registered_input_output_maps)
+                to_release_input_contents1 = self.trigger_release_inputs(updated_output_contents_full, work, registered_input_output_maps, final=True)
                 to_release_input_contents = to_release_input_contents + to_release_input_contents1
 
         to_resume_transform = False
